//Heap, implementation
//Week 12

#include "heap.h"

#include <algorithm>
#include <cmath>
#include <cassert>
using namespace std;

Heap::Heap() {
    size = 0;
}

void Heap::insertItem(int x) {

    // place x at end of tree, use percolate up to move to proper place
    size++;
    array[size] = x;
    percolateUp(size);
}

void Heap::percolateUp(int loc) {

    if (loc==1)  //base case, at root
        return;
    if (array[loc] > array[loc/2])   //base case, array[loc] > parent
        return;
    
    // recursive case: swap array[loc] with parent, repeat on the parent
    swap(array[loc],array[loc/2]);
    percolateUp(loc/2);

}

int Heap::removeMin() {
    assert(size>0);
    
    int temp = array[1];     // the minimum at root, return later
    array[1] = array[size];  // move last to root
    size--;
    percolateDown(1);
    return temp;
}

void Heap::percolateDown(int loc) {
    int left = loc*2;    //calculate index of left child
    int right = loc*2+1; //calculate index of right child
    
    if (left>size)  // base case, no children
        return;
    
    if (left==size) { // base case, one child (a leaf)
        if (array[loc]>array[left])
            swap(array[loc],array[left]);
        return;
    }
    
    //two children:
    if (array[left]>array[loc] && array[right]>array[loc])
        return;  //both children are greater than parent, done
    
    if (array[left] < array[right]) {  //left child is smaller
        swap(array[left],array[loc]);
        percolateDown(left);
    }
    else {                             //right child is smaller
        swap (array[right],array[loc]);
        percolateDown(right);
    }

}

void Heap::display () {
    for (int i=1; i<=size; i++) {
        if ((i & (i - 1)) == 0)  // i is a power of 2
            cout << endl;
        cout << array[i] << " " ;
    }
    cout << endl;
}
